@@ -1,4 +1,3 @@
-#! /usr/bin/python3.9
 "Python script containing functions and classes for general cryptographic use."
 
 # importing libraries
@@ -55,50 +54,123 @@ class UnknownError(Exception):
     pass
 
 
-def hash_(StringToHash:str, hash_code:str, return_type="hex"): # tested
+def hash_(ToHash, hash_code:str, return_hex=True, return_length = 256): # tested
     """
     Miscellenious function for implementing hash algorithms.
 
     Parameters:
+    `ToHash`: The bytes to be hashed; if not bytes, it will be converted to bytes.
     `hash_code`: A string indicating which hashing algorithm to use.
     currently supported hashes are:
     
-    SHA 224 / CODE = `'SHA224'`
-    
-    SHA 256 / CODE = `'SHA256'`
-    
-    SHA 512 / CODE = `'SHA512'`
-    
-    MD5 / CODE = `'MD5'`
+    `'SHA224'`: SHA224 hashing algorithm.
+    `'SHA256'`: SHA256 hashing algorithm.
+    `'SHA_384'`: SHA384 hashing algorithm.
+    `'SHA512'`: SHA512 hashing algorithm.
+    `'MD5'`: MD5 hashing algorithm.
+    `'SHA1'`: SHA1 hashing algorithm.
+    `'SHA3_224'`: SHA3_224 hashing algorithm.
+    `'SHA3_256'`: SHA3-256 hashing algorithm.
+    `'SHA3_384'`: SHA3-384 hashing algorithm.
+    `'SHA3_512'`: SHA3-512 hashing algorithm.
+    `'BLAKE2b'`: BLAKE2b hashing algorithm.
+    `'BLAKE2s'`: BLAKE2s hashing algorithm.
+    `'SHAKE_128'`: SHAKE_128 hashing algorithm.
+    `'SHAKE_256'`: SHAKE_256 hashing algorithm.
 
-    `return_type`: whether or not to return hex. specify 'bytes' if needed.
+
+    `return_hex`: A boolean indicating whether the output should be in hexadecimal or not.
+
+    `return_length`: An optional parameter specifying the amount of bytes to return. Used only in shake algorithms.
 
     Returns: a hash of the specific algorithm and data representation.
     """
+    ToHash = bytes(ToHash, 'utf-8')
+    hash_code = hash_code.upper()
     if hash_code == "SHA224":
-        hash_obj = hash.sha224(StringToHash.encode())
-        if return_type.lower() == "bytes":
+        hash_obj = hash.sha224(ToHash)
+        if return_hex is False:
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
-    if hash_code == "SHA256":
-        hash_obj = hash.sha256(StringToHash.encode())
-        if return_type.lower() == "bytes":
+    elif hash_code == "SHA256":
+        hash_obj = hash.sha256(ToHash)
+        if return_hex is False:
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
-    if hash_code == "SHA512":
-        hash_obj = hash.sha512(StringToHash.encode())
-        if return_type.lower() == "bytes":
+    elif hash_code == "SHA512":
+        hash_obj = hash.sha512(ToHash)
+        if return_hex is False:
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
-    if hash_code == "MD5":
-        hash_obj = hash.md5(StringToHash.encode())
-        if return_type.lower() == "bytes":
+    elif hash_code == "MD5":
+        hash_obj = hash.md5(ToHash)
+        if return_hex is False:
             return hash_obj.digest()
         else:
             return hash_obj.hexdigest()
+    elif hash_code == "SHA384":
+        hash_obj = hash.sha384(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "SHA1":
+        hash_obj = hash.sha1(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "BLAKE2B":
+        hash_obj = hash.blake2b(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "BLAKE2S":
+        hash_obj = hash.blake2s(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "SHA3_224":
+        hash_obj = hash.sha3_224(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "SHA3_256":
+        hash_obj = hash.sha3_256(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "SHA3_384":
+        hash_obj = hash.sha3_384(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "SHA3_512":
+        hash_obj = hash.sha3_512(ToHash)
+        if return_hex is False:
+            return hash_obj.digest()
+        else:
+            return hash_obj.hexdigest()
+    elif hash_code == "SHAKE_128":
+        hash_obj = hash.shake_128(ToHash)
+        if return_hex is False:
+            return hash_obj.digest(return_length)
+        else:
+            return hash_obj.hexdigest(return_length)
+    elif hash_code == "SHAKE_256":
+        hash_obj = hash.shake_256(ToHash)
+        if return_hex is False:
+            return hash_obj.digest(return_length)
+        else:
+            return hash_obj.hexdigest(return_length)
 
 
 def random_choice(given_list:list): # tested
@@ -240,7 +312,7 @@ def one_time_pad_encrypt(message:str, key:str):
     """
     A function for performing one time pad encryption. using a key generated from a keyphrase."""
     # generating the key
-    key_bytes = hash_(key, "SHA256", return_type="bytes")
+    key_bytes = hash_(key, "SHA256", return_hex=False)
     # getting the length of the key
     key_length = len(key_bytes)
     # getting the length of the message bytes
@@ -270,7 +342,7 @@ def one_time_pad_decrypt(message:bytes, key:str):
     """
     A function for performing one time pad decryption. using a key generated from a keyphrase."""
     # generating the key
-    key_bytes = hash_(key, "SHA256", return_type="bytes")
+    key_bytes = hash_(key, "SHA256", return_hex=False)
     # getting the length of the key
     key_length = len(key_bytes)
     # breaking the message bytes into chunks of the key length
@@ -1329,16 +1401,34 @@ def init_diagonostic():
     """A method for testing the various functions of this library."""
     print("Testing functions in the `cipher` module.")
     print ("testing hashes")
+    assert hash_("Hello World", "SHA224") == hash.sha224("Hello World".encode("utf-8")).hexdigest()
     assert hash_("Hello World", "SHA256") == hash.sha256("Hello World".encode("utf-8")).hexdigest()
     assert hash_("Hello World", "SHA512") == hash.sha512("Hello World".encode("utf-8")).hexdigest()
-    assert hash_("Hello World", "SHA224") == hash.sha224("Hello World".encode("utf-8")).hexdigest()
     assert hash_("Hello World", "MD5") == hash.md5("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA1") == hash.sha1("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA3_224") == hash.sha3_224("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA3_256") == hash.sha3_256("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA3_384") == hash.sha3_384("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHA3_512") == hash.sha3_512("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "BLAKE2b") == hash.blake2b("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "BLAKE2s") == hash.blake2s("Hello World".encode("utf-8")).hexdigest()
+    assert hash_("Hello World", "SHAKE_128") == hash.shake_128("Hello World".encode("utf-8")).hexdigest(256)
+    assert hash_("Hello World", "SHAKE_256") == hash.shake_256("Hello World".encode("utf-8")).hexdigest(256)
     print ("Hexdigests match")
     print("testing byte hashing")
-    assert hash_("Hello World", "SHA256", return_type="bytes") == hash.sha256("Hello World".encode("utf-8")).digest()
-    assert hash_("Hello World", "SHA512", return_type="bytes") == hash.sha512("Hello World".encode("utf-8")).digest()
-    assert hash_("Hello World", "SHA224", return_type="bytes") == hash.sha224("Hello World".encode("utf-8")).digest()
-    assert hash_("Hello World", "MD5", return_type="bytes") == hash.md5("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA224") == hash.sha224(b"Hello World").hexdigest()
+    assert hash_("Hello World", "SHA256", return_hex=False) == hash.sha256("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA512", return_hex=False) == hash.sha512("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "MD5", return_hex=False) == hash.md5("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA1", return_hex=False) == hash.sha1("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA3_224", return_hex=False) == hash.sha3_224("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA3_256", return_hex=False) == hash.sha3_256("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA3_384", return_hex=False) == hash.sha3_384("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHA3_512", return_hex=False) == hash.sha3_512("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "BLAKE2b", return_hex=False) == hash.blake2b("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "BLAKE2s", return_hex=False) == hash.blake2s("Hello World".encode("utf-8")).digest()
+    assert hash_("Hello World", "SHAKE_128", return_hex=False) == hash.shake_128("Hello World".encode("utf-8")).digest(256)
+    assert hash_("Hello World", "SHAKE_256", return_hex=False) == hash.shake_256("Hello World".encode("utf-8")).digest(256)
     print ("Bytes match")
     print ("testing key generation")
     print ("generating length 32 password")
@@ -1494,10 +1584,40 @@ def init_diagonostic():
     with open("test_file.txt", "r") as f:
         assert f.read() == "Hello World"
     # encrypting a string
-    encrypted_string = cipher.FerNet_encrypt_string(b"Hello World")
+    encrypted_string, decrypt_key = cipher.FerNet_encrypt_string(b"Hello World")
     # decrypting the string
-    decrypted_string = cipher.FerNet_decrypt_string(encrypted_string)
+    decrypted_string = cipher.FerNet_decrypt_string(encrypted_string, key=decrypt_key)
     # checking if the decrypted string is the same as the original
     assert decrypted_string.decode() == "Hello World"
+    # deleting all variables and test files
+    for i in globals():
+        del globals()[i]
+    for i in locals():
+        del locals()[i]
     print ("test complete; Fernet methods passed")
-    print ("All tests completed | All tests passed")
+    print ("All tests completed | General functionality tests passed")
+    print ("Beginning edge use case tests")
+    # testing functions first
+    print ("testing functions")
+    try:
+        # testing hash function
+        # obscene length
+        print ("testing hashing function")
+        print ("testing long length hash ability")
+        print ("resulting hash is: {}".format(hash_("Hello World"*2048, "SHA256")))
+        print ("testing hash function on bytes")
+        print ("resulting hash is: {}".format(hash_(b"Hello World", "SHA256")))
+        print ("Hash test finished")
+        pass
+    except Exception as e:
+        print (f"failure occurred during edge case test: {e}")
+    # deleting all variables
+    for i in globals():
+        del globals()[i]
+    for i in locals():
+        del locals()[i]
+    return 0
+
+
+if __name__ == "__main__":
+    init_diagonostic()
