@@ -1,7 +1,6 @@
-#! /usr/bin/python3.9
 from setuptools import setup, find_packages
 from Cython.Build import cythonize
-VERSION = "0.6.4"
+VERSION = "0.7.0"
 DESCRIPTION = "A module for encryption and information security operations."
 setup(
     name= "better cryptography",
@@ -19,7 +18,8 @@ setup(
         "Development Status :: 3 - Alpha",
         "Intended Audience :: Education",
         "Programming Language :: Python :: 3",
-        "Operating System :: POSIX :: Linux"
+        "Operating System :: POSIX :: Linux",
+        "License :: OSI Approved :: MIT License",
     ]
     
 )
\ No newline at end of file
